//******************************************************************************
//******************************************************************************
// Owner:   Andrew Forget (AndrewFo,x3438) 
//          Copyright ?1999, SOFTIMAGE, Avid Technologies
//------------------------------------------------------------------------------
//
// @doc     RICHEDITCTRLEX
//
// @module  RichEditCtrlEx.cpp |
//          This file implements the rich edit control extended class
//          <c CRichEditCtrlEx>.
//
// @version
//------------------------------------------------------------------------------
//.Version: Date:       Author:     Comments:
//.-------- ----------- ----------- --------------------------------------------
//.  1.0    01/18/99    AndrewFo    New file.
//******************************************************************************
//******************************************************************************


//******************************************************************************
//
// Includes
//
//******************************************************************************

#include "StdAfx.h"

#include "RichEditCtrlEx.h"


//******************************************************************************
//
// Debugging
//
//******************************************************************************

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//******************************************************************************
// Owner:   Andrew Forget (AndrewFo,x3438) 
//------------------------------------------------------------------------------
//
// @mfunc   | _AFX_RICHEDITEX_STATE | _AFX_RICHEDITEX_STATE |
//          Constructor for <c _AFX_RICHEDITEX_STATE>.
//
// @syntax  _AFX_RICHEDITEX_STATE() ;
//
// @version
//------------------------------------------------------------------------------
//.Version: Date:       Author:     Comments:
//.-------- ----------- ----------- --------------------------------------------
//.  1.0    01/18/99    AndrewFo    New code.
//******************************************************************************

_AFX_RICHEDITEX_STATE::_AFX_RICHEDITEX_STATE()
{
	m_hInstRichEdit20 = NULL ;
}

//******************************************************************************
// Owner:   Andrew Forget (AndrewFo,x3438) 
//------------------------------------------------------------------------------
//
// @mfunc   | _AFX_RICHEDITEX_STATE | ~_AFX_RICHEDITEX_STATE |
//          Destructor for <c _AFX_RICHEDITEX_STATE>.
//
// @syntax  ~_AFX_RICHEDITEX_STATE() ;
//
// @version
//------------------------------------------------------------------------------
//.Version: Date:       Author:     Comments:
//.-------- ----------- ----------- --------------------------------------------
//.  1.0    01/18/99    AndrewFo    New code.
//******************************************************************************

_AFX_RICHEDITEX_STATE::~_AFX_RICHEDITEX_STATE()
{
	if( m_hInstRichEdit20 != NULL )
	{
		::FreeLibrary( m_hInstRichEdit20 ) ;
	}
}


//******************************************************************************
//
// Global Data
//
//******************************************************************************

_AFX_RICHEDITEX_STATE _afxRichEditStateEx ;


//******************************************************************************
// Owner:   Andrew Forget (AndrewFo,x3438) 
//------------------------------------------------------------------------------
//
// @func    BOOL PASCAL | AfxInitRichEditEx |
//          This function must be called prior to creating any dialog which
//          uses the version 2.0 rich edit control.  For created classes, this
//          is called automatically.
//
// @syntax  BOOL PASCAL AfxInitRichEditEx() ;
//
// @rdesc   BOOL : A boolean value indicating:
// @flag        TRUE  | Success.
// @flag        FALSE | Failure.
//
// @comm    NOTE: This function will not work if AfxInitRichEdit() has been
//          called first and this will clash with the use of the CRichEditCtrl
//          class (including the CRichEditView class).
//
// @version
//------------------------------------------------------------------------------
//.Version: Date:       Author:     Comments:
//.-------- ----------- ----------- --------------------------------------------
//.  1.0    01/18/99    AndrewFo    New code.
//******************************************************************************

BOOL PASCAL AfxInitRichEditEx()
{
	if( ! ::AfxInitRichEdit() )
	{
		return FALSE ;
	}

	_AFX_RICHEDITEX_STATE* l_pState = &_afxRichEditStateEx ;

	if( l_pState->m_hInstRichEdit20 == NULL )
	{
		l_pState->m_hInstRichEdit20 = LoadLibraryA( "RICHED20.DLL" ) ;
	}

	return l_pState->m_hInstRichEdit20 != NULL ;
}



//******************************************************************************
// Owner:   Andrew Forget (AndrewFo,x3438) 
//------------------------------------------------------------------------------
//
// @mfunc   | CRichEditCtrlEx | CRichEditCtrlEx |
//          Constructor for <c CRichEditCtrlEx>.
//
// @syntax  CRichEditCtrlEx() ;
//
// @version
//------------------------------------------------------------------------------
//.Version: Date:       Author:     Comments:
//.-------- ----------- ----------- --------------------------------------------
//.  1.0    01/18/99    AndrewFo    New code.
//******************************************************************************

CRichEditCtrlEx::CRichEditCtrlEx()
{
	m_nCursor = CURSOR_IBEAM;
}

//******************************************************************************
// Owner:   Andrew Forget (AndrewFo,x3438) 
//------------------------------------------------------------------------------
//
// @mfunc   | CRichEditCtrlEx | ~CRichEditCtrlEx |
//          Destructor for <c CRichEditCtrlEx>.
//
// @syntax  ~CRichEditCtrlEx() ;
//
// @version
//------------------------------------------------------------------------------
//.Version: Date:       Author:     Comments:
//.-------- ----------- ----------- --------------------------------------------
//.  1.0    01/18/99    AndrewFo    New code.
//******************************************************************************

CRichEditCtrlEx::~CRichEditCtrlEx()
{
}


//******************************************************************************
// Owner:   Andrew Forget (AndrewFo,x3438) 
//------------------------------------------------------------------------------
//
// @msgmap  CRichEditCtrl | CRichEditCtrlEx |
//          Message map for <c CRichEditCtrlEx>.
//
// @version
//------------------------------------------------------------------------------
//.Version: Date:       Author:     Comments:
//.-------- ----------- ----------- --------------------------------------------
//.  1.0    01/18/99    AndrewFo    New code.
//******************************************************************************

BEGIN_MESSAGE_MAP( CRichEditCtrlEx, CRichEditCtrl )

	//{{AFX_MSG_MAP( CRichEditCtrlEx )
	//}}AFX_MSG_MAP

//	ON_WM_MOUSEWHEEL()
	ON_WM_CONTEXTMENU()
	ON_WM_SETCURSOR()

	ON_MESSAGE(WM_PASTE, OnPasteOnlyText)

	ON_COMMAND(ID_CHAT_INPUT_COPY, &CRichEditCtrlEx::OnMenuCopy)
	ON_COMMAND(ID_CHAT_INPUT_CUT, &CRichEditCtrlEx::OnMenuCut)
	ON_COMMAND(ID_CHAT_INPUT_PASTE, &CRichEditCtrlEx::OnMenuPaste)
	ON_WM_KEYDOWN()
END_MESSAGE_MAP()


//******************************************************************************
// Owner:   Andrew Forget (AndrewFo,x3438) 
//------------------------------------------------------------------------------
//
// @mfunc   BOOL | CRichEditCtrlEx | Create |
//          This method is used to create the version 2.0 rich edit control.
//
// @syntax  BOOL Create(
//          			DWORD		in_dwStyle, 
//          			const RECT& in_rcRect, 
//          			CWnd*		in_pParentWnd, 
//          			UINT		in_nID ) ;
//
// @parm    DWORD | in_dwStyle |
//          The styles for the rich edit control.
//
// @parm    const RECT& | in_rcRect |
//          The rectangle for the control window.
//
// @parm    CWnd* | in_pParentWnd |
//          Pointer to the parent window.
//
// @parm    UINT | in_nID |
//          The dialog control ID for the control.
//
// @rdesc   BOOL : A boolean value indicating:
// @flag        TRUE  | Sucess.
// @flag        FALSE | Failure.
//
// @version
//------------------------------------------------------------------------------
//.Version: Date:       Author:     Comments:
//.-------- ----------- ----------- --------------------------------------------
//.  1.0    01/18/99    AndrewFo    New code.
//******************************************************************************

BOOL CRichEditCtrlEx::Create(
			DWORD		in_dwStyle, 
			const RECT& in_rcRect, 
			CWnd*		in_pParentWnd, 
			UINT		in_nID )
{
	if( ! ::AfxInitRichEditEx() )
	{
		return FALSE ;
	}

	CWnd* l_pWnd = this ;

	return l_pWnd->Create( _T( "RichEdit20A" ), NULL, in_dwStyle, in_rcRect, in_pParentWnd, in_nID ) ;
}

BOOL CRichEditCtrlEx::PreTranslateMessage(MSG* pMsg)
{
	if (pMsg->message == WM_COPY)
	{
		Copy();
		return TRUE;
	}
	else if (pMsg->message == WM_PASTE)
	{
		OnPasteOnlyText(0, 0);
		return TRUE;
	}
	else if (pMsg->message == WM_MOUSEHWHEEL)
	{
		return TRUE;
	}

	return CRichEditCtrl::PreTranslateMessage(pMsg);
}

void CRichEditCtrlEx::OnContextMenu(CWnd* pWnd, CPoint point)
{
	m_nCursor = CURSOR_DEFAULT;

	CPoint pt(0, 0);
	GetCursorPos(&pt);

	CMenu menuList;
	menuList.LoadMenu(IDR_MENU_CHAT);

	CMenu* popupMenu = menuList.GetSubMenu(2);

	if (GetSelText().IsEmpty())
	{
		popupMenu->EnableMenuItem(ID_CHAT_INPUT_CUT , MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
		popupMenu->EnableMenuItem(ID_CHAT_INPUT_COPY , MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
	}

	popupMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, this);

	m_nCursor = CURSOR_IBEAM;
}

void CRichEditCtrlEx::OnMenuCopy()
{
	Copy();
}

void CRichEditCtrlEx::OnMenuCut()
{
	Cut();
}

void CRichEditCtrlEx::OnMenuPaste()
{
	OnPasteOnlyText(0, 0);
}

BOOL CRichEditCtrlEx::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
	switch (m_nCursor)
	{
	case CURSOR_DEFAULT:
		::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
		return TRUE;
	case CURSOR_IBEAM:
		::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_IBEAM));
		return TRUE;
	}

	return CWnd::OnSetCursor(pWnd, nHitTest, message);
}

LRESULT CRichEditCtrlEx::OnPasteOnlyText(WPARAM wParam, LPARAM lParam)
{
	//  ϰ  ؽƮ Ѵ.
	PasteSpecial(CF_TEXT);
	return 0L;
}

LRESULT CRichEditCtrlEx::OnCanPaste(WPARAM w, LPARAM)
{
	if (w == 0) 
	{ // Ŭ忡 ִ  ϳİ  
		return ::IsClipboardFormatAvailable(CF_TEXT); // Ŭ忡 TEXT ִ 
	}

	return (w == CF_TEXT) ? TRUE : FALSE;
}

void CRichEditCtrlEx::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if (GetKeyState(VK_CONTROL) < 0)
	{
		// ȭâ ʿ Ű ´
		if (_T('r') == nChar || _T('R') == nChar ||	// Right alignment
			_T('e') == nChar || _T('E') == nChar ||	// Center alignment
			_T('j') == nChar || _T('J') == nChar)	// Justify alignment
		{
			return;
		}
		else if (_T('v') == nChar || _T('V') == nChar)
		{
			// ⺻ RichEditCtrl Paste  Ư   
			//  ִ ڿ ٿ   ϰ ȭâ ٿֱ ؼ̴.
			Paste();
			return;
		}

		if (GetKeyState(VK_SHIFT) < 0)
		{
			if (_T('0') <= nChar && nChar <= _T('9'))
			{
				DWORD dwIndex = nChar - _T('0') + 10;	// Ctrl+Shift ε 10 
				InsertAutoText(dwIndex);
			}
		}
		else if (_T('0') <= nChar && nChar <= _T('9'))
		{
			DWORD dwIndex = nChar - _T('0');
			InsertAutoText(dwIndex);
		}
	}
	else if (nChar == VK_INSERT)
	{
		if (GetAsyncKeyState(VK_SHIFT) & 0x8000)
		{
			Paste();
			return;
		}
	}

	CRichEditCtrl::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CRichEditCtrlEx::InsertAutoText(DWORD dwIndex)
{
	if (dwIndex < 0 || 20 <= dwIndex)
		return;

	ReplaceSel(GET_CONFIG()->m_vecAutoText[dwIndex]);
}

